<?php

namespace App\Http\Controllers\Administrator\Package;
use App\Classes\Models\Package\Package;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;

class IndexController extends Controller
{

    protected $packageObj;

    public function __construct( Package $package )
    {
        $this->packageObj = $package;
    }

    public function index( Request $request )
    {
        $page = 0;
        $search = '';
        if ( $request->get( 'page' ) ) 
        {
            $page = $request->get( 'page' );
        }

        if ( $request->get( 'search' ) ) 
        {
            $search = trim( $request->get( 'search' ) );
        }
        
        $packages = $this->packageObj->list( $search, $page );
        $totalRecordCount = $this->packageObj->listTotalCount( $search );

        $basePath = \Request::url() . '?search=' . $search . '&';
        $paging = $this->packageObj->preparePagination( $totalRecordCount, $basePath );
        return view( 'administrator.package.index', compact( 'packages', 'paging' ) );
    }

    public function save( Request $request )
    {
        $submitData = $request->all();
        $data = $submitData;
        // dd($data);
        
        $result = $this->packageObj->saveRecord( $data );
        if ( isset( $submitData['id'] ) ) 
        {
            $package = $this->packageObj->display( $result['id'] );
            if ( $result['success'] == false ) 
            {
                return view( 'administrator.package.create', compact('package' ) )->withErrors( $result['message'] );
            } 
            else 
            {
                $request->session()->flash( 'success', $result['message'] );
                return view( 'administrator.package.create', compact('package' ) );
            }
        } 
        else 
        {
            if ( $result['success'] == false ) 
            {
                return view( 'administrator.package.create')->withErrors( $result['message'] );
            } 
            else 
            {
                
                return redirect()->route( 'administrator.packages.index' )->with( 'success', 'プランを正常に追加しました。' );
            }
        }
    }

    public function create()
    {
        $package_status = 'active';
        return view( 'administrator.package.create', compact('package_status') );
    }

    public function edit( $id )
    {
        $package = $this->packageObj->display( $id );
        $package_status = $package->status; 
        return view( 'administrator.package.create', compact( 'package', 'package_status') );
    }

    public function destroy( $id )
    {
        $isdelete = $this->packageObj->removed( $id );
        if ( $isdelete ) {
            return redirect()->route( 'administrator.packages.index' )->with( 'success', 'プランを削除しました。' );
        } 
        else 
        {
            return redirect()->route( 'administrator.packages.index' )->with( 'error', 'エラーが発生しました。' );
        }
    }

    public function packagestatusupdate()
    {
        $packages = Package::where('status', 'active')->get();
        
        foreach ($packages as $package) 
        {
            $package_id         = $package->id;
            $start_date         = $package->created_at->toDateTimeString();
            $expires_no_month   = $package->expires_no_month;
            $expire_date        = date('Y-m-d H:i:s',strtotime($start_date.' +'.$expires_no_month.' month'));
            
            $current_date = date("Y-m-d H:i:s");
            if ( strtotime($current_date) > strtotime($expire_date) )
            {
                $packages = $this->packageObj->statusupdate( $package_id );
            }
        }
    }
}
