<?php

namespace App\Http\Controllers\Administrator\User;

use App\Classes\Models\User\User;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;
use App\Classes\Models\Package\Package;
use App\Classes\Models\Subscription\Subscription;
use Stripe;

class IndexController extends Controller
{

    protected $userObj;
    protected $packageObj;
    protected $subscriptionObj;

    public function __construct( User $user )
    {
        
        Stripe\Stripe::setApiKey(env('STRIPE_SECRET'));
        $this->userObj = $user;
        $this->packageObj = new Package();
        $this->subscriptionObj = new Subscription();
        
    }

    public function index( Request $request )
    {
       

        $page = 0;
        $search = '';
        if ( $request->get( 'page' ) ) {
            $page = $request->get( 'page' );
        }
        if ( $request->get( 'search' ) ) {
            $search = trim( $request->get( 'search' ) );
        }
        
        $users = $this->userObj->list( $search, $page );
        
        $totalRecordCount = $this->userObj->listTotalCount( $search );

        $basePath = \Request::url() . '?search=' . $search . '&';
        $paging = $this->userObj->preparePagination( $totalRecordCount, $basePath );
        
        return view( 'administrator.users.index', compact( 'users', 'paging' ) );
    }

    public function save( Request $request )
    {
        $submitData = $request->all();
        $data = $submitData;
        if ($request->package_id == '0') {
            $data['package_id'] = '';
            $data['interval'] = '';
        }
        // dd($data);
        $result = $this->userObj->saveRecord( $data );
        
        if ( isset( $data['id'] ) ) {
            $user = $this->userObj->display( $result['id'] );
            if ( $result['success'] == false ) {
                return back()->withErrors( $result['message'] );
            } else {

                
                
                $request->session()
                        ->flash( 'success', $result['message'] );
                return  redirect()->route( 'administrator.users.index' )->with( 'success', 'ユーザー情報を変更しました。' );
            }
        } else {
            if ( $result['success'] == false ) {
                return back()->withErrors( $result['message'] );
            } else {

                
                $request->session()
                        ->flash( 'success', $result['message'] );
                return  redirect()->route( 'administrator.users.index' )->with( 'success', 'ユーザーを追加しました。' );
            }
        }
    }

    public function create()
    {
        $packages = $this->packageObj->alllist( '', 0 );
        
        return view( 'administrator.users.create', compact( 'packages' ));
    }

    

    

    public function edit( $id )
    {
        $user = $this->userObj->display( $id );

        $packages = $this->packageObj->alllist( '', 0 );
        $current_subscription = $this->subscriptionObj->activeSubscription( $id, 1 ); 

        return view( 'administrator.users.create', compact( 'user','packages','current_subscription' ) );
    }

    public function destroy( $id )
    {
    
        
        $isdelete = $this->userObj->removed( $id );
        if ( $isdelete ) {
            $this->subscriptionObj->remove_by_userId( $id ); 
            return redirect()
                ->route( 'administrator.users.index' )
                ->with( 'success', '削除が完了しました。' );
        } else {
            return redirect()
                ->route( 'administrator.users.index' )
                ->with( 'error', 'エラーが発生しました。' );
        }
    }

    public function check_subscription(Request $request)
    {
        
        $subscription_id = $request->subscription_id;
        $sub_primary_id = $request->sub_primary_id;
        $subscription = \Stripe\Subscription::retrieve(
            $subscription_id
        );
        $result = $subscription->status;
        if($subscription->status !== "active"){

           $update_data = array();
            $update_data['status'] = 0;
            $update_data['sub_primary_id'] = $sub_primary_id;
            $update_data['canceled_at'] = $subscription->canceled_at;
            $result = $this->subscriptionObj->updateRecord($update_data,$subscription_id);
        }

        
        

        

        return $result;
    }

}
