<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Carbon\Carbon;
use App\Classes\Helpers\Home\Helper;
use App\Classes\Models\Package\Package;
use App\Classes\Models\Subscription\Subscription;
use App\Classes\Models\User\User;
use Auth;
use Stripe;


class HomeController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */


    protected $packageObj;
    protected $subscriptionObj;
    protected $userObj;

    public function __construct()
    {
        Stripe\Stripe::setApiKey(env('STRIPE_SECRET'));
        $this->middleware('auth');
        $this->packageObj = new Package();
        $this->subscriptionObj = new Subscription();
        $this->userObj = new User();

    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $data['page_title'] = '課金メニュー';
        $data['packages'] = $this->packageObj->alllist( '', 0 ); 
        $data['current_subscription'] = $this->subscriptionObj->activeSubscription( Auth::user()->id, 1 ); 
        $data['user'] = $this->userObj->display( Auth::user()->id );
        
        return view('home',$data);
    }

    public function subscription(Request $request)
    {


        if(isset($request->is_upgrade) && $request->is_upgrade == "1"){
            // print_r('expression');
            //     die();
            $result = $this->cancel_subscription($request->cur_subscription_id,$request->sub_primary_id);
            
            if($result['id'] ){
                
            }else{
                $request->session()->flash('error', 'エラーが発生しました' );
                return back();
            }
        }

        $customerObj = $this->create_customer($request);
        $productObj = $this->create_product($request);
        $priceObj = $this->create_price($request,$productObj->id);
        $paymentObj = $this->create_subscription($customerObj->id,$priceObj->id);
        
        $data = array();
        $data['user_id'] = Auth::user()->id;
        $data['package_id'] = $request->plan_id;
        $data['holder_name'] = $request->holder_name;
        $data['interval'] = $request->plan_period == 'monthly'? 1:2;
        $data['customer_id'] = $paymentObj->customer;
        $data['product_id'] = $paymentObj->items->data[0]->price->product;
        $data['price_id'] = $paymentObj->items->data[0]->price->id;
        $data['subscription_id'] = $paymentObj->id;
        $data['current_period_start'] = $paymentObj->current_period_start;
        $data['current_period_end'] = $paymentObj->current_period_end;
        $data['status'] = $paymentObj->status == 'active'? 1:0;

        $result = $this->subscriptionObj->saveRecord( $data );

        $user_data = array();
        $user_data['id'] =  $data['user_id'];
        $user_data['package_id'] = $data['package_id'];
        $user_data['interval'] = $data['interval'];
        $user_data['has_subscription'] = $data['status'];
        $result_user = $this->userObj->updateRecord($user_data,$data['user_id']);

        
        if(isset( $result['id'] ) && $result['id'] != ''){
            $request->session()->flash('success', $result['message'] );
        }else{
            $request->session()->flash('error', 'エラーが発生しました' );
        }
        
          
        return back();
    }

    public function create_customer($request)
    {
        Stripe\Stripe::setApiKey(env('STRIPE_SECRET'));
        $customer = \Stripe\Customer::create([
            "description" => "user_id: " . Auth::user()->id,
            "email" => Auth::user()->email,
            "source" => $request->stripe_token,
        ]);

        return $customer;
    }

    public function create_product($request)
    {
        $package = $this->packageObj->display( $request->plan_id );
        $product = \Stripe\Product::create([
            "name" => $package->plan_name.":".$package->plan_description,
        ]);

        return $product;
    }

    public function create_price($request,$product_id)
    {
        
        $price = \Stripe\Price::create([
            "unit_amount" => $request->price,
            "currency" => 'jpy',
            'recurring' => ['interval' => substr($request->plan_period,0,-2)],
            'product' => $product_id,
        ]);

        return $price;
    }

    public function create_subscription($customer_id,$price_id)
    {
        
        $subscription = \Stripe\Subscription::create([
            "customer" => $customer_id,
            'items' => [
                    ['price' => $price_id],
              ],
        ]);

        return $subscription;
    }

    public function cancel_payment(Request $request)
    {
        $sub_primary_id = $request->sub_primary_id;
        $subscription_id = $request->subscription_id;
        
        $result = $this->cancel_subscription($subscription_id,$sub_primary_id);
        

        return $result;
    }

    public function cancel_subscription($subscription_id,$sub_primary_id)
    {
        
        $subscription = \Stripe\Subscription::retrieve(
            $subscription_id
        );
        if($subscription->status == "active"){
           $subscription->cancel();
        }

        $update_data = array();
        $update_data['status'] = 0;
        $update_data['sub_primary_id'] = $sub_primary_id;
        $update_data['canceled_at'] = $subscription->canceled_at;
        

        $result = $this->subscriptionObj->updateRecord($update_data,$subscription_id);

        
        

        return $result;
    }

    public function identification(Request $request)
    {
        $holder_name = $request->holder_name;
        $holder_name = preg_replace('/\s+/', '', $holder_name);
        $user_id = Auth::user()->id; 
        $user = $this->userObj->display( $user_id );
        $user_name = $user->first_name.$user->last_name;
        
        if($holder_name == $user_name){
            $update_data = array();
            $update_data['id'] =  $user_id;
            $update_data['identification'] =  1;
            $result = $this->userObj->updateRecord($update_data,$user_id);
            return 'confirm';
        }else{
            return 'failed';
        }
        

        
    }

}
