<?php

namespace App\Classes\Models\User;

use App\Classes\Models\BaseModel;

class User extends BaseModel
{
    protected $table = 'users';
    protected $primaryKey = 'id';
    
    protected $per_page = '10';

    protected $entity = 'users';
    protected $searchableColumns = ['first_name','last_name','email'];
    protected $fillable = ['id',
                           'uuid',
                           'first_name',
                           'last_name',
                           'first_kana',
                           'last_kana',
                           'password',
                           'email',
                           'status','identification','package_id','interval','has_subscription'];
   

    public function __construct( array $attributes = [] )
    {
        $this->bootIfNotBooted();
        $this->syncOriginal();
        $this->fill( $attributes );
        
    }

    public function list( $search = '', $page = 0, $sortedBy = ['id'], $sortedOrder = ['DESC'] )
    {
        if($search == '')
        {
            $per_page = $this->per_page;
            $list = $this->setSelect()
                        ->addSearch( $search )
                        ->addOrderBy( $sortedBy, $sortedOrder )
                        ->addPaging( $page, $per_page )

                        ->leftjoin('package', 'users.package_id', '=', 'package.id')
                        ->select( 'users.*','package.plan_name')
                        ->orderBy( 'id', 'DESC' )
                        ->get();
            return $list;
        }
        else
        {
            $per_page = $this->per_page;
            $list = $this->setSelect()
                        ->addSearch( $search )
                        ->addOrderBy( $sortedBy, $sortedOrder )
                        ->addPaging( $page, $per_page )
                        ->addGroupBy( $this->table . '.id' )
                        
                        ->get();
            return $list;
        }
    }

    
    public function getUserName($user_id)
    {
        $user = $this->setSelect()->where('id', $user_id)->get()->first();
        $firstName = @$user->first_name;
        $lastName = @$user->last_name;                      
        // dd($user_id, $firstName, $lastName);
        return $firstName.' '.$lastName;
    }
    
    public function checkUser($user_id)
    {
         $count = $this->setSelect()
                      ->where('id', $user_id)
                      ->where('status', 1)
                      ->get()
                      ->count();
        return $count;
    }

    public function addGroupBy( $groupByName = null )
    {
        if ( ! empty( $groupByName ) ) {

            $this->queryBuilder->groupBy( $groupByName );
        }
        return $this;
    }

    public function addOrderBy( $sortedBy = [], $sortedOrder = [] )
    {

        if ( ! empty( $sortedBy ) && ! is_array( $sortedBy ) ) {
            $sortedBy = [$sortedBy];
        }

        if ( ! empty( $sortedOrder ) && ! is_array( $sortedOrder ) ) {
            $sortedOrder = [$sortedOrder];
        }

        if ( ! empty( $sortedBy ) && ! empty( $sortedOrder ) ) {

            foreach ( $sortedBy as $key => $sortedByName ) {

                $sortedOrder = ! empty( $sortedOrder[$key] ) ? $sortedOrder[$key] : $sortedOrder[0];

                $this->queryBuilder->orderBy( $sortedByName, $sortedOrder );
            }
        }
        return $this;
    }

    public function listTotalCount( $search = '', $sortedBy = ['id'], $sortedOrder = ['DESC'] )
    {
        $this->reset();
        if($search == '')
        {
            $count = $this->setSelect()
                      ->addSearch( $search )
                      ->addOrderBy( $sortedBy, $sortedOrder )
                      ->addGroupBy( $this->table . '.id' )
                      ->get()
                      ->count();

            return $count;
        }
        else
        {
            $count = $this->setSelect()
                      ->addSearch( $search )
                      ->addOrderBy( $sortedBy, $sortedOrder )
                      ->addGroupBy( $this->table . '.id' )
                    //   ->where('role_id', 2)
                      ->get()
                      ->count();

            return $count;
        }
    }

    public function preparePagination( $total, $basePath )
    {
        $perpage = $this->per_page;
        $pageHelper = new \App\Classes\Helpers\PageHelper( $perpage, 'page' );
        $pageHelper->set_total( $total );
        $pageHelper->page_links( $basePath );
        return $pageHelper->page_links( $basePath );
    }

    public function saveRecord( $data )
    {
        $rules = ['first_name' => 'required|max:100',
                  'last_name'  => 'required|max:100',
                  'email'      => 'required|unique:' . $this->table . ',email',];
        if ( isset( $data['id'] ) && $data['id'] != '' ) {
            $id = $data['id'];

            $rules['email'] = 'required|unique:' . $this->table . ',email,' . $id . ',id';

            if ( ! empty( $data['password'] ) ) {
                $rules['password'] = 'required|confirmed';
                $rules['password_confirmation'] = 'required';
            }
        } else {
            $rules['password'] = 'required|confirmed';
            $rules['password_confirmation'] = 'required';
        }

        $validationResult = $this->validateData( $rules, $data );
        $result = [];
        $result['id'] = '';
        if ( $validationResult['success'] == false ) {
            $result['success'] = false;
            $result['message'] = $validationResult['message'];
            $result['id'] = $data['id'];
            return $result;
        }

        $link_ids = [];
        
        if ( ! empty( $data['password'] ) ) {
            $data['password'] = bcrypt( $data['password'] );
        } else {
            unset( $data['password'] );
        }
        $this->beforeSave( $data );
        if ( isset( $data['id'] ) && $data['id'] != '' ) {
            $user = \App\Classes\Models\User\User::findOrFail( $data['id'] );
            $user->update( $data );
            $this->afterSave( $data, $user );
            
            $result['id'] = $user->id;

        } else {
            $user = \App\Classes\Models\User\User::create( $data );
            $result['id'] = $user->id;
            $this->afterSave( $data, $user );
        }
        
        $result['success'] = true;
        $result['message'] = "ユーザーが正常に保存されました。";
        return $result;
    }

    public function removed( $user_id )
    {
        $this->beforeRemoved( $user_id );
        $deleteMemberObj = $this->display( $user_id );
        if ( ! empty( $deleteMemberObj ) ) {
            $delete = $deleteMemberObj->delete();
            $this->afterRemoved( $user_id );
            return $delete;
        }
        return false;
    }

    public function display( $user_id )
    {
        return $this->load( $user_id );
    }

    public function load( $user_id )
    {

        $this->beforeLoad( $user_id );

        $return = $this->setSelect()
                       ->addUserIdFilter( $user_id )
                       ->get()
                       ->first();

        $this->afterLoad( $user_id, $return );

        return $return;
    }

    public function addUserIdFilter( $user_id = 0 )
    {
        $this->queryBuilder->where( 'id', $user_id );
        return $this;
    }

    public function addStatusFilter( $status = 0 )
    {
        if($status > 0) {
            $this->queryBuilder->where( 'status', $status );
        }
        return $this;
    }

    public function updateRecord($data,$where_id)
    {
        
        $user = \App\Classes\Models\User\User::findOrFail($data['id']);
        
        $user->update( $data );
        
        $result['id'] = $user->id;
        return $result;
    }
}